@@ -12,6 +12,7 @@ import torch.nn as nn
 from gym.spaces import Space
 from ray.rllib import Policy
 from ray.rllib import SampleBatch
+from ray.rllib.agents import Trainer
 from ray.rllib.models.action_dist import ActionDistribution
 from ray.rllib.models.model import flatten
 from ray.rllib.models.model import restore_original_dimensions
@@ -21,12 +22,10 @@ from ray.tune.logger import pretty_print
 from torch import Tensor
 from torch.optim import Optimizer
 
-from raylab.agents import Trainer
-from raylab.modules.catalog import get_module
 from raylab.pytorch.utils import convert_to_tensor
 from raylab.utils.dictionaries import deep_merge
 
-from .action_dist import WrapModuleDist
+from .modules import get_module
 from .optimizer_collection import OptimizerCollection
 
 
@@ -34,6 +33,8 @@ class TorchPolicy(Policy):
     """A Policy that uses PyTorch as a backend.
 
     Attributes:
+        dist_class: Action distribution class for computing actions. Must be set
+            by subclasses before calling `__init__`.
         device: Device in which the parameter tensors reside. All input samples
             will be converted to tensors and moved to this device
         module: The policy's neural network module. Should be compilable to
@@ -49,10 +50,11 @@ class TorchPolicy(Policy):
         config = deep_merge(
             {**self.get_default_config(), "worker_index": None},
             config,
-            new_keys_allowed=False,
-            whitelist=Trainer._allow_unknown_subkeys,
-            override_all_if_type_changes=Trainer._override_all_subkeys_if_type_changes,
+            new_keys_allowed=True,
+            override_all_if_type_changes=self._override_all_subkeys_if_type_changes,
         )
+        # Allow subclasses to set `dist_class` before calling init
+        action_dist = getattr(self, "dist_class", None)
         super().__init__(observation_space, action_space, config)
 
         self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
@@ -64,10 +66,31 @@ class TorchPolicy(Policy):
             self.optimizers[name] = optimizer
 
         # === Policy attributes ===
-        self.dist_class = WrapModuleDist
+        self.dist_class = action_dist
+        self.dist_class.check_model_compat(self.module)
         self.framework = "torch"  # Needed to create exploration
         self.exploration = self._create_exploration()
 
+    @property
+    def model(self):
+        """The policy's NN module.
+
+        Mostly for compatibility with RLlib's API.
+        """
+        return self.module
+
+    @property
+    def _override_all_subkeys_if_type_changes(self) -> List[str]:
+        """Nested config dictionaries with special "type" keys.
+
+        Returns:
+            List of top level keys with value=dict, for which we always simply
+            override the entire value (dict), iff the "type" key in that value
+            dict changes.
+        """
+        # pylint:disable=protected-access
+        return Trainer._override_all_subkeys_if_type_changes + ["module"]
+
     @staticmethod
     @abstractmethod
     def get_default_config() -> dict:
@@ -142,7 +165,9 @@ class TorchPolicy(Policy):
             self.convert_to_tensor([1]),
         )
 
+        # pylint:disable=not-callable
         action_dist = self.dist_class(dist_inputs, self.module)
+        # pylint:enable=not-callable
         actions, logp = self.exploration.get_exploration_action(
             action_distribution=action_dist, timestep=timestep, explore=explore
         )
@@ -243,7 +268,9 @@ class TorchPolicy(Policy):
             state_batches,
             self.convert_to_tensor([1]),
         )
+        # pylint:disable=not-callable
         action_dist = self.dist_class(dist_inputs, self.module)
+        # pylint:enable=not-callable
         log_likelihoods = action_dist.logp(input_dict[SampleBatch.ACTIONS])
         return log_likelihoods
 
